import 'dart:convert';
import 'dart:io';

import 'package:musicus_shared/musicus_shared.dart';

import 'server_library.dart';

class Server {
  static const version = '0.0.1';

  final ServerLibrary library;

  Server(String path) : library = ServerLibrary(path);

  Future<void> respond(HttpRequest request) async {
    request.response.headers.add('Access-Control-Allow-Origin', '*');

    switch (request.method) {
      case 'POST':
        await post(request);
        break;
      case 'PUT':
        await put(request);
        break;
      case 'DELETE':
        await delete(request);
        break;
      case 'GET':
        await get(request);
        break;
      default:
        request.response.statusCode = HttpStatus.methodNotAllowed;
    }
    request.response.close();
  }

  Future<void> post(HttpRequest request) async {
    final path = request.uri.pathSegments;
    final content = await request.transform(utf8.decoder).join();

    if (path.length == 1) {
      switch (path[0]) {
        case 'eras':
          final id = await library.addEra(Era.fromString(content));
          request.response.write(id);
          break;
        case 'musicians':
          final id = await library.addMusician(Musician.fromString(content));
          request.response.write(id);
          break;
        case 'genres':
          final id = await library.addGenre(Genre.fromString(content));
          request.response.write(id);
          break;
        case 'keys':
          final id = await library.addKey(Key.fromString(content));
          request.response.write(id);
          break;
        case 'works':
          final id = await library.addWork(Work.fromString(content));
          request.response.write(id);
          break;
        default:
          request.response.statusCode = HttpStatus.notFound;
      }
    } else {
      request.response.statusCode = HttpStatus.notFound;
    }
  }

  Future<void> put(HttpRequest request) async {
    final path = request.uri.pathSegments;
    final content = await request.transform(utf8.decoder).join();

    if (path.length == 2) {
      try {
        final id = int.parse(path[1]);
        switch (path[0]) {
          case 'eras':
            library.updateEra(id, Era.fromString(content));
            break;
          case 'musicians':
            library.updateMusician(id, Musician.fromString(content));
            break;
          case 'genres':
            library.updateGenre(id, Genre.fromString(content));
            break;
          case 'keys':
            library.updateKey(id, Key.fromString(content));
            break;
          case 'works':
            library.updateWork(id, Work.fromString(content));
            break;
          default:
            request.response.statusCode = HttpStatus.notFound;
        }
      } on FormatException {
        request.response.statusCode = HttpStatus.notFound;
      }
    } else {
      request.response.statusCode = HttpStatus.notFound;
    }
  }

  Future<void> delete(HttpRequest request) async {
    final path = request.uri.pathSegments;

    if (path.length == 2) {
      try {
        final id = int.parse(path[1]);
        switch (path[0]) {
          case 'eras':
            library.deleteEra(id);
            break;
          case 'musicians':
            library.deleteMusician(id);
            break;
          case 'genres':
            library.deleteGenre(id);
            break;
          case 'keys':
            library.deleteKey(id);
            break;
          case 'works':
            library.deleteWork(id);
            break;
          default:
            request.response.statusCode = HttpStatus.notFound;
        }
      } on FormatException {
        request.response.statusCode = HttpStatus.notFound;
      }
    } else {
      request.response.statusCode = HttpStatus.notFound;
    }
  }

  Future<void> get(HttpRequest request) async {
    final path = request.uri.pathSegments;

    if (path.length == 1) {
      switch (path[0]) {
        case 'version':
          request.response.write(version);
          break;
        case 'eras':
          final ids = await library.getEras();
          request.response.write(jsonEncode(ids.toList()));
          break;
        case 'musicians':
          final ids = await library.getMusicians();
          request.response.write(jsonEncode(ids.toList()));
          break;
        case 'genres':
          final ids = await library.getGenres();
          request.response.write(jsonEncode(ids.toList()));
          break;
        case 'keys':
          final ids = await library.getKeys();
          request.response.write(jsonEncode(ids.toList()));
          break;
        default:
          request.response.statusCode = HttpStatus.notFound;
      }
    } else if (path.length == 2) {
      try {
        final id = int.parse(path[1]);
        switch (path[0]) {
          case 'eras':
            request.response.write(library.getWork(id));
            break;
          case 'musicians':
            request.response.write(library.getMusician(id));
            break;
          case 'genres':
            request.response.write(library.getGenre(id));
            break;
          case 'keys':
            request.response.write(library.getKey(id));
            break;
          case 'works':
            request.response.write(library.getWork(id));
            break;
          default:
            request.response.statusCode = HttpStatus.notFound;
        }
      } on FormatException {
        request.response.statusCode = HttpStatus.notFound;
      }
    } else {
      request.response.statusCode = HttpStatus.notFound;
    }
  }
}
